/*
   microMVC Java Port by Donald Stinchfield <donald.stinchfield@microMVC.org>, et al.
   microMVC - Copyright(c) 2006-08 Futurescale, Inc., Some rights reserved.
   Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

package com.btc.micromvc.core.controller;

import com.btc.micromvc.core.view.View;
import com.btc.micromvc.interfaces.IControllerCmd;
import com.btc.micromvc.interfaces.IController;
import com.btc.micromvc.interfaces.INotificationListener;
import com.btc.micromvc.interfaces.INotification;
import com.btc.micromvc.patterns.observer.Observer;

import java.util.Hashtable;

/**
 * A Singleton <code>IController</code> implementation.
 * 
 * <P>
 * In microMVC, the <code>Controller</code> class follows the 'Command and
 * Controller' strategy, and assumes these responsibilities:
 * <UL>
 * <LI> Remembering which <code>IControllerCmd</code>s are intended to handle which
 * <code>INotifications</code>.</LI>
 * <LI> Registering itself as an <code>IObserver</code> with the
 * <code>View</code> for each <code>INotification</code> that it has an
 * <code>IControllerCmd</code> mapping for.</LI>
 * <LI> Creating a new instance of the proper <code>IControllerCmd</code> to handle
 * a given <code>INotification</code> when notified by the <code>View</code>.</LI>
 * <LI> Calling the <code>IControllerCmd</code>'s <code>execute</code> method,
 * passing in the <code>INotification</code>.</LI>
 * </UL>
 * 
 * <P>
 * Your application must register <code>IControllerCmds</code> with the Controller.
 * <P>
 * The simplest way is to subclass </code>Facade</code>, and use its <code>initializeController</code>
 * method to add your registrations.
 * 
 * @see com.btc.micromvc.core.view.View View
 * @see com.btc.micromvc.patterns.observer.Observer Observer
 * @see com.btc.micromvc.patterns.observer.Notification Notification
 * @see com.btc.micromvc.patterns.command.SimpleCommand SimpleCommand
 * @see com.btc.micromvc.patterns.command.MacroCommand MacroCommand
 */
public class Controller implements IController
{

	/** 
	 * Singleton instance
	 */
	protected static Controller instance;

	/**
	 * Mapping of Notification names to Command Class references
	 */
	protected Hashtable commandMap;

	/**
	 * Local reference to View
	 */
	protected View view;

	/**
	 * Constructor.
	 * 
	 * <P>
	 * This <code>IController</code> implementation is a Singleton, so you
	 * should not call the constructor directly, but instead call the static
	 * Singleton Factory method <code>Controller.getInstance()</code>
	 * 
	 * @throws Error
	 *             Error if Singleton instance has already been constructed
	 * 
	 */
	protected Controller( )
	{
		instance = this;
		this.commandMap = new Hashtable();
		initializeController();
	}

	/**
	 * Initialize the Singleton <code>Controller</code> instance.
	 * 
	 * <P>
	 * Called automatically by the constructor.
	 * </P>
	 * 
	 * <P>
	 * Note that if you are using a subclass of <code>View</code> in your
	 * application, you should <i>also</i> subclass <code>Controller</code>
	 * and override the <code>initializeController</code> method in the
	 * following way:
	 * </P>
	 * 
	 * <listing> // ensure that the Controller is talking to my IView
	 * implementation override public function initializeController( ) : void {
	 * view = MyView.getInstance(); } </listing>
	 * 
	 */
	protected void initializeController( )
	{
		this.view = View.getInstance();
	}

	/**
	 * <code>Controller</code> Singleton Factory method.
	 * 
	 * @return the Singleton instance of <code>Controller</code>
	 */
	public synchronized static Controller getInstance( )
	{
		if (instance == null) {
			instance = new Controller();
		}
		return instance;
	}

	/**
	 * If an <code>IControllerCmd</code> has previously been registered to handle a
	 * the given <code>INotification</code>, then it is executed.
	 * 
	 * @param note
	 *            an <code>INotification</code>
	 */
	public void executeCommand( INotification note )
	{
		Class commandClassRef = (Class) this.commandMap.get( note.getName() );
		if (commandClassRef == null) {
			return;
		}
		try {
			IControllerCmd commandInstance = (IControllerCmd) commandClassRef.newInstance();
			commandInstance.execute( note );
		} catch (IllegalAccessException iae) {
		} catch (InstantiationException ie) {
		}
	}

	/**
	 * Register a particular <code>IControllerCmd</code> class as the handler for a
	 * particular <code>INotification</code>.
	 * 
	 * <P>
	 * If an <code>IControllerCmd</code> has already been registered to handle
	 * <code>INotification</code>s with this name, it is no longer used, the
	 * new <code>IControllerCmd</code> is used instead.
	 * </P>
	 * 
	 * The Observer for the new IControllerCmd is only created if this the 
	 * first time an IControllerCmd has been regisered for this Notification name.
	 * 
	 * @param notificationName
	 *            the name of the <code>INotification</code>
	 * @param commandClassRef
	 *            the <code>Class</code> of the <code>IControllerCmd</code>
	 */
	public void registerCommand( String notificationName, Class commandClassRef )
	{
		if (null != this.commandMap.put( notificationName, commandClassRef )) return;
		this.view.registerObserver( notificationName, new Observer( new INotificationListener()
		{
			public void onNotification( INotification notification )
			{
				executeCommand( notification );
			}
		}, this ) );
	}

	/**
	 * Remove a previously registered <code>IControllerCmd</code> to
	 * <code>INotification</code> mapping.
	 * 
	 * @param notificationName
	 *            the name of the <code>INotification</code> to remove the
	 *            <code>IControllerCmd</code> mapping for
	 */
	public void removeCommand( String notificationName )
	{
		this.commandMap.remove( notificationName );
	}

}
