/*
   microMVC Java Port by Donald Stinchfield <donald.stinchfield@microMVC.org>, et al.
   microMVC - Copyright(c) 2006-08 Futurescale, Inc., Some rights reserved.
   Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

package com.btc.micromvc.core.model;

import java.util.Hashtable;

import com.btc.micromvc.interfaces.IModel;
import com.btc.micromvc.interfaces.IModelService;

/**
 * A Singleton <code>IModel</code> implementation.
 * 
 * <P>
 * In microMVC, the <code>Model</code> class provides access to model objects
 * (Proxies) by named lookup.
 * 
 * <P>
 * The <code>Model</code> assumes these responsibilities:
 * </P>
 * 
 * <UL>
 * <LI>Maintain a cache of <code>IModelService</code> instances.</LI>
 * <LI>Provide methods for registering, retrieving, and removing
 * <code>IModelService</code> instances.</LI>
 * </UL>
 * 
 * <P>
 * Your application must register <code>IModelService</code> instances with the
 * <code>Model</code>. Typically, you use an <code>IControllerCmd</code> to
 * create and register <code>IModelService</code> instances once the
 * <code>Facade</code> has initialized the Core actors.
 * </p>
 * 
 * @see com.btc.micromvc.patterns.ModelService.ModelService ModelService
 * @see com.btc.micromvc.interfaces.IModelService IModelService
 */
public class Model implements IModel
{

	/**
	 * Singleton instance
	 */ 
	protected static Model instance;

	/**
	 * Mapping of ModelServiceNames to IModelService instances
	 */
	protected Hashtable ModelServiceMap;

	/**
	 * Constructor.
	 * 
	 * <P>
	 * This <code>IModel</code> implementation is a Singleton, so you should
	 * not call the constructor directly, but instead call the static Singleton
	 * Factory method <code>Model.getInstance()</code>
	 * 
	 * @throws Error
	 *             Error if Singleton instance has already been constructed
	 * 
	 */
	protected Model( )
	{
		instance = this;
		this.ModelServiceMap = new Hashtable();
		initializeModel();
	}

	/**
	 * Initialize the Singleton <code>Model</code> instance.
	 * 
	 * <P>
	 * Called automatically by the constructor, this is your opportunity to
	 * initialize the Singleton instance in your subclass without overriding the
	 * constructor.
	 * </P>
	 * 
	 */
	protected void initializeModel( )
	{
	}

	/**
	 * <code>Model</code> Singleton Factory method.
	 * 
	 * @return the Singleton instance
	 */
	public synchronized static Model getInstance( )
	{
		if (instance == null) {
			instance = new Model();
		}
		return instance;
	}

	/**
	 * Register an <code>ModelService</code> with the <code>Model</code>.
	 * 
	 * @param ModelService
	 *            an <code>ModelService</code> to be held by the <code>Model</code>.
	 */
	public void registerModelService( IModelService ModelService )
	{
		this.ModelServiceMap.put( ModelService.getModelServiceName(), ModelService );
	}

	/**
	 * Remove an <code>ModelService</code> from the <code>Model</code>.
	 * 
	 * @param ModelService
	 *            name of the <code>ModelService</code> instance to be removed.
	 */
	public IModelService removeModelService( String ModelService )
	{
		return (IModelService) this.ModelServiceMap.remove( ModelService );
	}

	/**
	 * Retrieve an <code>ModelService</code> from the <code>Model</code>.
	 * 
	 * @param ModelService
	 * @return the <code>ModelService</code> instance previously registered with the
	 *         given <code>ModelServiceName</code>.
	 */
	public IModelService retrieveModelService( String ModelService )
	{
		return (IModelService) this.ModelServiceMap.get( ModelService );
	}
}
