/*
   microMVC Java Port by Donald Stinchfield <donald.stinchfield@microMVC.org>, et al.
   microMVC - Copyright(c) 2006-08 Futurescale, Inc., Some rights reserved.
   Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

package com.btc.micromvc.interfaces;

/**
 * The interface definition for a microMVC Facade.
 * 
 * <P>
 * The Facade Pattern suggests providing a single class to act as a central
 * point of communication for a subsystem.
 * </P>
 * 
 * <P>
 * In microMVC, the Facade acts as an interface between the core MVC actors
 * (Model, View, Controller) and the rest of your application.
 * </P>
 * 
 * @see com.btc.micromvc.interfaces.IModel IModel
 * @see com.btc.micromvc.interfaces.IView IView
 * @see com.btc.micromvc.interfaces.IController IController
 * @see com.btc.micromvc.interfaces.IControllerCmd IControllerCmd
 * @see com.btc.micromvc.interfaces.INotification INotification
 */
public interface IApplicationFacade extends INotifier
{
	/**
	 * Notify <code>Observer</code>s of an <code>INotification</code>.
	 * 
	 * @param note
	 *            the <code>INotification</code> to have the <code>View</code>
	 *            notify observers of.
	 */
	public void notifyObservers( INotification note );

	/**
	 * Register an <code>IModelService</code> with the <code>Model</code> by name.
	 * 
	 * @param ModelService
	 *            the <code>IModelService</code> to be registered with the
	 *            <code>Model</code>.
	 */
	public void registerModelService( IModelService ModelService );

	/**
	 * Retrieve a <code>IModelService</code> from the <code>Model</code> by name.
	 * 
	 * @param ModelServiceName
	 *            the name of the <code>IModelService</code> instance to be
	 *            retrieved.
	 * @return the <code>IModelService</code> previously regisetered by
	 *         <code>ModelServiceName</code> with the <code>Model</code>.
	 */
	public IModelService retrieveModelService( String ModelServiceName );

	/**
	 * Remove an <code>IModelService</code> instance from the <code>Model</code> by
	 * name.
	 * 
	 * @param ModelServiceName
	 *            the <code>IModelService</code> to remove from the
	 *            <code>Model</code>.
	 */
	public IModelService removeModelService( String ModelServiceName );

	/**
	 * Register an <code>IControllerCmd</code> with the <code>Controller</code>.
	 * 
	 * @param noteName
	 *            the name of the <code>INotification</code> to associate the
	 *            <code>IControllerCmd</code> with.
	 * @param commandClassRef
	 *            a reference to the <code>Class</code> of the
	 *            <code>IControllerCmd</code>.
	 */
	public void registerCommand( String noteName, Class commandClassRef );

	/**
	 * Remove a previously registered <code>IControllerCmd</code> to <code>INotification</code> mapping from the Controller.
	 * 
	 * @param notificationName the name of the <code>INotification</code> to remove the <code>IControllerCmd</code> mapping for
	 */
	public void removeCommand( String notificationName);

	/**
	 * Register an <code>IViewMediator</code> instance with the <code>View</code>.
	 * 
	 * @param mediator
	 *            a reference to the <code>IViewMediator</code> instance
	 */
	public void registerMediator( IViewMediator mediator );

	/**
	 * Retrieve an <code>IViewMediator</code> instance from the <code>View</code>.
	 * 
	 * @param mediatorName
	 *            the name of the <code>IViewMediator</code> instance to retrievve
	 * @return the <code>IViewMediator</code> previously registered with the given
	 *         <code>mediatorName</code>.
	 */
	public IViewMediator retrieveMediator( String mediatorName );

	/**
	 * Remove a <code>IViewMediator</code> instance from the <code>View</code>.
	 * 
	 * @param mediatorName
	 *            name of the <code>IViewMediator</code> instance to be removed.
	 */
	public IViewMediator removeMediator( String mediatorName );
}
