/*
   microMVC Java Port by Donald Stinchfield <donald.stinchfield@microMVC.org>, et al.
   microMVC - Copyright(c) 2006-08 Futurescale, Inc., Some rights reserved.
   Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

package com.btc.micromvc.interfaces;

/**
 * The interface definition for a microMVC Controller.
 * 
 * <P>
 * In microMVC, an <code>IController</code> implementor follows the 'Command
 * and Controller' strategy, and assumes these responsibilities:
 * <UL>
 * <LI> Remembering which <code>IControllerCmd</code>s are intended to handle which
 * <code>INotifications</code>.</LI>
 * <LI> Registering itself as an <code>IObserver</code> with the
 * <code>View</code> for each <code>INotification</code> that it has an
 * <code>IControllerCmd</code> mapping for.</LI>
 * <LI> Creating a new instance of the proper <code>IControllerCmd</code> to handle
 * a given <code>INotification</code> when notified by the <code>View</code>.</LI>
 * <LI> Calling the <code>IControllerCmd</code>'s <code>execute</code> method,
 * passing in the <code>INotification</code>.</LI>
 * </UL>
 * 
 * @see com.btc.micromvc.interfaces INotification
 * @see com.btc.micromvc.interfaces IControllerCmd
 */
public interface IController
{

	/**
	 * Register a particular <code>IControllerCmd</code> class as the handler for a
	 * particular <code>INotification</code>.
	 * 
	 * @param notificationName
	 *            the name of the <code>INotification</code>
	 * @param commandClassRef
	 *            the Class of the <code>IControllerCmd</code>
	 */
	public void registerCommand( String notificationName, Class commandClassRef );

	/**
	 * Execute the <code>IControllerCmd</code> previously registered as the handler
	 * for <code>INotification</code>s with the given notification name.
	 * 
	 * @param notification
	 *            the <code>INotification</code> to execute the associated
	 *            <code>IControllerCmd</code> for
	 */
	public void executeCommand( INotification notification );

	/**
	 * Remove a previously registered <code>IControllerCmd</code> to
	 * <code>INotification</code> mapping.
	 * 
	 * @param notificationName
	 *            the name of the <code>INotification</code> to remove the
	 *            <code>IControllerCmd</code> mapping for
	 */
	public void removeCommand( String notificationName );
}
