/*
   microMVC Java Port by Donald Stinchfield <donald.stinchfield@microMVC.org>, et al.
   microMVC - Copyright(c) 2006-08 Futurescale, Inc., Some rights reserved.
   Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

package com.btc.micromvc.patterns.facade;

import com.btc.micromvc.core.controller.Controller;
import com.btc.micromvc.core.model.Model;
import com.btc.micromvc.core.view.View;
import com.btc.micromvc.interfaces.IApplicationFacade;
import com.btc.micromvc.interfaces.IViewMediator;
import com.btc.micromvc.interfaces.INotification;
import com.btc.micromvc.interfaces.IModelService;
import com.btc.micromvc.patterns.observer.Notification;


/**
 * A base Singleton <code>IApplicationFacade</code> implementation.
 * 
 * <P>
 * In microMVC, the <code>Facade</code> class assumes these responsibilities:
 * <UL>
 * <LI>Initializing the <code>Model</code>, <code>View</code> and
 * <code>Controller</code> Singletons.</LI>
 * <LI>Providing all the methods defined by the <code>IModel, 
 * IView, & IController</code>
 * interfaces.</LI>
 * <LI>Providing the ability to override the specific <code>Model</code>,
 * <code>View</code> and <code>Controller</code> Singletons created.</LI>
 * <LI>Providing a single point of contact to the application for registering
 * <code>Commands</code> and notifying <code>Observers</code></LI>
 * </UL>
 * <P>
 * Example usage: <listing> import org.microMVC.patterns.facade.&lowast;;
 * 
 * import com.me.myapp.model.~~; import com.me.myapp.view.~~; import
 * com.me.myapp.controller.~~;
 * 
 * public class MyFacade extends Facade { // Notification constants. The Facade
 * is the ideal // location for these constants, since any part // of the
 * application participating in microMVC // Observer Notification will know the
 * Facade. public static const GO_COMMAND:String = "go"; // Override Singleton
 * Factory method public static function getInstance() : MyFacade { if (instance ==
 * null) instance = new MyFacade(); return instance as MyFacade; } // optional
 * initialization hook for Facade override public function initializeFacade() :
 * void { super.initializeFacade(); // do any special subclass initialization
 * here } // optional initialization hook for Controller override public
 * function initializeController() : void { // call super to use the microMVC
 * Controller Singleton. super.initializeController(); // Otherwise, if you're
 * implmenting your own // IController, then instead do: // if ( controller !=
 * null ) return; // controller = MyAppController.getInstance(); // do any
 * special subclass initialization here // such as registering Commands
 * registerCommand( GO_COMMAND, com.me.myapp.controller.GoCommand ) } //
 * optional initialization hook for Model override public function
 * initializeModel() : void { // call super to use the microMVC Model Singleton.
 * super.initializeModel(); // Otherwise, if you're implmenting your own //
 * IModel, then instead do: // if ( model != null ) return; // model =
 * MyAppModel.getInstance(); // do any special subclass initialization here //
 * such as creating and registering Model ModelServices // that don't require a facade
 * reference at // construction time, such as fixed type lists // that never
 * need to send Notifications. regsiterModelService( new USStateNamesModelService() ); //
 * CAREFUL: Can't reference Facade instance in constructor // of new ModelServices from
 * here, since this step is part of // Facade construction! Usually, ModelServices
 * needing to send // notifications are registered elsewhere in the app // for
 * this reason. } // optional initialization hook for View override public
 * function initializeView() : void { // call super to use the microMVC View
 * Singleton. super.initializeView(); // Otherwise, if you're implmenting your
 * own // IView, then instead do: // if ( view != null ) return; // view =
 * MyAppView.getInstance(); // do any special subclass initialization here //
 * such as creating and registering Mediators // that do not need a Facade
 * reference at construction // time. registerMediator( new LoginMediator() ); //
 * CAREFUL: Can't reference Facade instance in constructor // of new Mediators
 * from here, since this is a step // in Facade construction! Usually, all
 * Mediators need // receive notifications, and are registered elsewhere in //
 * the app for this reason. } } </listing>
 * 
 * @see com.btc.micromvc.core.model.Model Model
 * @see com.btc.micromvc.core.view.View View
 * @see com.btc.micromvc.core.controller.Controller Controller
 * @see com.btc.micromvc.patterns.observer.Notification Notification
 * @see com.btc.micromvc.patterns.mediator.ViewMediator Mediator
 * @see com.btc.micromvc.patterns.ModelService.ModelService ModelService
 * @see com.btc.micromvc.patterns.command.SimpleCommand SimpleCommand
 * @see com.btc.micromvc.patterns.command.MacroCommand MacroCommand
 */
public class ApplicationFacade implements IApplicationFacade
{

	/**
	 * The Singleton instance of the Facade
	 */
	protected static ApplicationFacade instance = null;

	/**
	 * Reference to the Controller
	 */
	protected Controller controller = null;

	/**
	 * Reference to the Model
	 */
	protected Model model = null;

	/**
	 * Reference to the View
	 */
	protected View view = null;

	/**
	 * Constructor.
	 * 
	 * <P>
	 * This <code>IApplicationFacade</code> implementation is a Singleton, so you should
	 * not call the constructor directly, but instead call the static Singleton
	 * Factory method <code>Facade.getInstance()</code>
	 * 
	 * @throws Error
	 *             Error if Singleton instance has already been constructed
	 * 
	 */
	protected ApplicationFacade( )
	{
		if (instance != null) throw new RuntimeException("Facade already constructed");
		initializeFacade();
	}

	/**
	 * Initialize the Singleton <code>Facade</code> instance.
	 * 
	 * <P>
	 * Called automatically by the constructor. Override in your subclass to do
	 * any subclass specific initializations. Be sure to call
	 * <code>super.initializeFacade()</code>, though.
	 * </P>
	 */
	protected void initializeFacade( )
	{
		initializeModel();
		initializeController();
		initializeView();
	}

	/**
	 * Facade Singleton Factory method
	 * 
	 * @return the Singleton instance of the Facade
	 */
	public synchronized static ApplicationFacade getInstance( )
	{
		if (instance == null) {
			try {
				instance = new ApplicationFacade();
			} catch (Exception e) {
			}
		}
		return instance;
	}

	/**
	 * Initialize the <code>Controller</code>.
	 * 
	 * <P>
	 * Called by the <code>initializeFacade</code> method. Override this
	 * method in your subclass of <code>Facade</code> if one or both of the
	 * following are true:
	 * <UL>
	 * <LI> You wish to initialize a different <code>IController</code>.</LI>
	 * <LI> You have <code>Commands</code> to register with the
	 * <code>Controller</code> at startup.</code>. </LI>
	 * </UL>
	 * If you don't want to initialize a different <code>IController</code>,
	 * call <code>super.initializeController()</code> at the beginning of your
	 * method, then register <code>Command</code>s.
	 * </P>
	 */
	protected void initializeController( )
	{
		if (this.controller != null) {
			return;
		}
		this.controller = Controller.getInstance();
	}

	/**
	 * Initialize the <code>Model</code>.
	 * 
	 * <P>
	 * Called by the <code>initializeFacade</code> method. Override this
	 * method in your subclass of <code>Facade</code> if one or both of the
	 * following are true:
	 * <UL>
	 * <LI> You wish to initialize a different <code>IModel</code>.</LI>
	 * <LI> You have <code>ModelService</code>s to register with the Model that do
	 * not retrieve a reference to the Facade at construction time.</code></LI>
	 * </UL>
	 * If you don't want to initialize a different <code>IModel</code>, call
	 * <code>super.initializeModel()</code> at the beginning of your method,
	 * then register <code>ModelService</code>s.
	 * <P>
	 * Note: This method is <i>rarely</i> overridden; in practice you are more
	 * likely to use a <code>Command</code> to create and register <code>ModelService</code>s
	 * with the <code>Model</code>, since <code>ModelService</code>s with mutable
	 * data will likely need to send <code>INotification</code>s and thus
	 * will likely want to fetch a reference to the <code>Facade</code> during
	 * their construction.
	 * </P>
	 */
	protected void initializeModel( )
	{
		if (this.model != null) {
			return;
		}
		this.model = Model.getInstance();
	}

	/**
	 * Initialize the <code>View</code>.
	 * 
	 * <P>
	 * Called by the <code>initializeFacade</code> method. Override this
	 * method in your subclass of <code>Facade</code> if one or both of the
	 * following are true:
	 * <UL>
	 * <LI> You wish to initialize a different <code>IView</code>.</LI>
	 * <LI> You have <code>Observers</code> to register with the
	 * <code>View</code></LI>
	 * </UL>
	 * If you don't want to initialize a different <code>IView</code>, call
	 * <code>super.initializeView()</code> at the beginning of your method,
	 * then register <code>IViewMediator</code> instances.
	 * <P>
	 * Note: This method is <i>rarely</i> overridden; in practice you are more
	 * likely to use a <code>Command</code> to create and register
	 * <code>Mediator</code>s with the <code>View</code>, since
	 * <code>IViewMediator</code> instances will need to send
	 * <code>INotification</code>s and thus will likely want to fetch a
	 * reference to the <code>Facade</code> during their construction.
	 * </P>
	 */
	protected void initializeView( )
	{
		if (this.view != null) {
			return;
		}
		this.view = View.getInstance();
	}

	/**
	 * Register an <code>IControllerCmd</code> with the <code>Controller</code> by
	 * Notification name.
	 * 
	 * @param noteName
	 *            the name of the <code>INotification</code> to associate the
	 *            <code>IControllerCmd</code> with
	 * @param commandClassRef
	 *            a reference to the Class of the <code>IControllerCmd</code>
	 */
	public void registerCommand( String noteName, Class commandClassRef )
	{
		this.controller.registerCommand( noteName, commandClassRef );
	}

	/**
	 * Remove a previously registered <code>IControllerCmd</code> to <code>INotification</code> mapping from the Controller.
	 * 
	 * @param notificationName the name of the <code>INotification</code> to remove the <code>IControllerCmd</code> mapping for
	 */
	public void removeCommand( String notificationName ) {
		this.controller.removeCommand( notificationName );
	}


	/**
	 * Register a <code>IViewMediator</code> with the <code>View</code>.
	 * 
	 * @param mediator
	 *            the name to associate with this <code>IViewMediator</code>
	 */
	public void registerMediator( IViewMediator mediator )
	{
		if (this.view != null) {
			this.view.registerMediator( mediator );
		}
	}

	/**
	 * Register an <code>IModelService</code> with the <code>Model</code> by name.
	 * 
	 * @param ModelService
	 *            the name of the <code>IModelService</code> instance to be
	 *            registered with the <code>Model</code>.
	 */
	public void registerModelService( IModelService ModelService )
	{
		this.model.registerModelService( ModelService );
	}

	/**
	 * Remove an <code>IViewMediator</code> from the <code>View</code>.
	 * 
	 * @param mediatorName
	 *            name of the <code>IViewMediator</code> to be removed.
	 * @return the <code>IViewMediator</code> that was removed from the <code>View</code>	             
	 */
	public IViewMediator removeMediator( String mediatorName )
	{
		if (this.view != null) {
			return this.view.removeMediator( mediatorName );
		}
		return null;
	}

	/**
	 * Remove an <code>IModelService</code> from the <code>Model</code> by name.
	 * 
	 * @param ModelServiceName
	 *            the <code>IModelService</code> to remove from the
	 *            <code>Model</code>.
      * @return the <code>IModelService</code> that was removed from the <code>Model</code>	             
	 */
	public IModelService removeModelService( String ModelServiceName )
	{
		if (this.model != null) {
			return this.model.removeModelService( ModelServiceName );
		}
		return null;
	}

	/**
	 * Retrieve an <code>IViewMediator</code> from the <code>View</code>.
	 * 
	 * @param mediatorName
	 * @return the <code>IViewMediator</code> previously registered with the given
	 *         <code>mediatorName</code>.
	 */
	public IViewMediator retrieveMediator( String mediatorName )
	{
		return this.view.retrieveMediator( mediatorName );
	}

	/**
	 * Retrieve an <code>IModelService</code> from the <code>Model</code> by name.
	 * 
	 * @param ModelServiceName
	 *            the name of the ModelService to be retrieved.
	 * @return the <code>IModelService</code> instance previously registered with the
	 *         given <code>ModelServiceName</code>.
	 */
	public IModelService retrieveModelService( String ModelServiceName )
	{
		return this.model.retrieveModelService( ModelServiceName );
	}

	/**
	 * Create and send an <code>INotification</code>.
	 * 
	 * <P>
	 * Keeps us from having to construct new notification 
	 * instances in our implementation code.
	 * @param notificationName the name of the notiification to send
	 * @param body the body of the notification (optional)
	 * @param type the type of the notification (optional)
	 */ 
	public void sendNotification( String notificationName, Object body, String type ) 
	{
		notifyObservers( new Notification( notificationName, body, type ) );
	}

	/**
	 * Notify <code>Observer</code>s.
	 * 
	 * @param notification
	 *            the <code>INotification</code> to have the <code>View</code>
	 *            notify <code>Observers</code> of.
	 */
	public void notifyObservers( INotification notification )
	{
		if (this.view != null) {
			this.view.notifyObservers( notification );
		}
	}


}
